# servant-xml-conduit

This is similar to <https://hackage.haskell.org/package/servant-xml>,
however we use
[`xml-conduit`](https://hackage.haskell.org/package/xml-conduit)
instead of [`xmlbf`](https://hackage.haskell.org/package/xmlbf). Also,
we only provide a way to serialize
[`Document`](https://hackage.haskell.org/package/xml-conduit-1.9.1.3/docs/Text-XML.html#t:Document)
leaving end user to implement his own
[`FromXml`](https://hackage.haskell.org/package/xmlbf-0.7/docs/Xmlbf.html#t:FromXml)
typeclass.

An example of using such a typeclass would be:
```haskell
import Data.Text qualified as T
import Text.XML qualified as XML
import Text.XML.Cursor qualified as XMLCursor


data MyCustomErrors =
     ErrorNoElementFound Text XMLCursor.Cursor
   | ...

class FromXML a where
  fromXML :: (Functor m, Applicative m, MonadError MyCustomErrors m) => XMLCursor.Cursor -> m a

instance FromXML T.Text where
  fromXML = pure . T.concat . XMLCursor.content
  
instance FromXML Int where
  fromXML c = do
    let txt = (T.concat . XMLCursor.content) c
	
	case readEither txt of
	  Left err -> throwError $ ErrorParsingInt c $ show err
	  Right i  -> pure i
```
